﻿
Imports System.Net
Imports System.IO
Imports System.Data.SqlTypes
Imports System.Text
Imports System.Text.RegularExpressions
'Imports Newtonsoft.Json.Linq

Public Class OdataLinkCLRProcessor

    Public Enum ForceTLSEnum
        TLS11 = 768
        TLS12 = 3072
        TLS13 = 12288
    End Enum


    'this function returns and collates all the pages from a starting url.
    Public Function GetOdataPages(ByVal strURL As String, ByVal strUserName As String, ByVal strPassword As String) As String

        Dim strAuthorisation As String
        Dim strNextPageLink As String

        Dim strPageResult As String

        Dim objResultBuilder As StringBuilder = Nothing
        Dim strResult As String = ""

        'initialise the url
        strNextPageLink = strURL

        'create the authorisation header string if applicable
        strAuthorisation = CreateAuthorisation(strUserName, strPassword)


        'stringbuilder to append the results
        objResultBuilder = New StringBuilder()

        Do

            'fetch the page
            strPageResult = GetOdataPage(strNextPageLink, strAuthorisation)

            If (Not strPageResult Is Nothing) AndAlso (strPageResult.Length > 0) Then
                'if a page was returned, append it
                'and fetch the next page link.

                Call AppendResult(objResultBuilder, strPageResult)

                strNextPageLink = GetNextPageLink(strPageResult)
            Else
                'if no page was return, do not get the next page link and exit loop
                strNextPageLink = Nothing
            End If

        Loop While Not strNextPageLink Is Nothing

        'return the results
        strResult = objResultBuilder.ToString()
        
        Return strResult

    End Function

    Public Function GetOdataPage(ByVal strURL As String, ByVal strUserName As String, ByVal strPassword As String) As String

        Dim strAuthorisation As String
        Dim strResult As String

        'create the authorisation header string if applicable
        strAuthorisation = CreateAuthorisation(strUserName, strPassword)

        'fetch the page
        strResult = GetOdataPage(strURL, strAuthorisation)

        'return the result
        Return strResult

    End Function

    Private Function GetOdataPage(ByVal strURL As String, ByVal strAuthorisation As String) As String

        Dim objRequest As WebRequest
        Dim objResponse As WebResponse
        Dim objDataStream As Stream
        Dim objDataReader As StreamReader

        Dim strPagedResult As String

        'init various https parameters including tls 1.2
        ServicePointManager.Expect100Continue = True
        ServicePointManager.SecurityProtocol = CType(ForceTLSEnum.TLS12, SecurityProtocolType)
        ServicePointManager.DefaultConnectionLimit = 9999

        'create the request
        objRequest = HttpWebRequest.Create(strURL)


        'append the authorisation header if specified
        If (Not strAuthorisation Is Nothing) AndAlso (strAuthorisation.Length > 0) Then
            objRequest.Headers.Add("Authorization", strAuthorisation)
        End If


        'read the response and return it
        objResponse = objRequest.GetResponse()

        objDataStream = objResponse.GetResponseStream()
        objDataReader = New StreamReader(objDataStream)

        strPagedResult = objDataReader.ReadToEnd()

        Return strPagedResult

    End Function



    'this function appends the result to the buffer for multipages
    Private Sub AppendResult(ByVal objResult As StringBuilder, ByVal strDocument As String)

        Dim strSubDocument As String = ""
        Dim strResult As String

        Dim intInsertAtLast As Int32

        Try
            If objResult.Length = 0 Then
                'if no results already in buffer, append it
                Call objResult.Append(strDocument)
            Else

                'if results are in the buffer, we need to insert the latest page in the correct spot

                'first parse the sub document to get the value node only
                strSubDocument = ParseSubDocument(strDocument)

                If (Not strSubDocument Is Nothing) AndAlso (strSubDocument.Length > 0) Then

                    'get the string of the current buffer
                    strResult = objResult.ToString()

                    'find where the insert the new value nodes
                    'it must be after the last ]} characters
                    intInsertAtLast = strResult.LastIndexOf("}")
                    If intInsertAtLast > 0 Then
                        intInsertAtLast = strResult.LastIndexOf("]", intInsertAtLast)


                        If intInsertAtLast > 0 Then
                            'once we have found the spot
                            'identify if we were an empty value array or not (e.g. "value": [])

                            If strResult.Substring(intInsertAtLast - 1, 1) = "[" Then

                                'if we we were, the  insert
                                objResult.Insert(intInsertAtLast, strSubDocument)
                            Else

                                'if we weren't, then append a comma and insert
                                objResult.Insert(intInsertAtLast, ", ")
                                objResult.Insert(intInsertAtLast + 2, strSubDocument)
                            End If
                        End If
                    End If
                End If
            End If
        Catch ex As Exception
        End Try

    End Sub






    'this function extracts the value nodes only for multiple pages
    Private Function ParseSubDocument(ByVal strDocument As String) As String
        Dim strRegex As String
        Dim objRegex As Regex
        Dim objMatch As Match
        Dim objGroup As Group

        Dim intFirst As Int32
        Dim intLast As Int32

        Dim strSubDocument As String = ""

        Try
            If (Not strDocument Is Nothing) AndAlso (strDocument.Length > 0) Then

                'regex to get the "value": [ element
                strRegex = "(""value""\s*:\s*\[\s*)"

                objRegex = New Regex(strRegex)

                objMatch = objRegex.Match(strDocument)

                'if we have a match, then parse it
                If Not objMatch Is Nothing Then
                    objGroup = objMatch.Groups(0)
                    intFirst = objGroup.Index + objGroup.Length

                    If intFirst > 0 Then

                        'the value node is always the last element, we need to find the ending ]} characters and not include them
                        intLast = strDocument.LastIndexOf("}")
                        If intLast > 0 Then
                            intLast = strDocument.LastIndexOf("]", intLast)
                        End If

                        If intFirst > 0 And intLast > 0 Then
                            strSubDocument = strDocument.Substring(intFirst, intLast - intFirst)
                        End If
                    End If

                End If
            End If

        Catch ex As Exception

        End Try

        Return strSubDocument

    End Function



    'this function gets the next page link for multipage
    Private Function GetNextPageLink(ByVal strDocument As String) As String

        Dim strRegex As String
        Dim objRegex As Regex
        Dim objMatch As Match

        Dim strNextLink As String = Nothing

        Try
            If (Not strDocument Is Nothing) AndAlso (strDocument.Length > 0) Then

                'regex to get the "@odata.nextlink": "value"
                strRegex = "(""@odata\.nextLink""\s*:\s*"")(.+)("",\s*""value"")"

                objRegex = New Regex(strRegex)

                objMatch = objRegex.Match(strDocument)

                If Not objMatch Is Nothing Then
                    strNextLink = objMatch.Groups(2).Value

                    If (Not strNextLink Is Nothing) Then
                        strNextLink = strNextLink.Trim

                        If strNextLink.Length = 0 Then
                            strNextLink = Nothing
                        End If
                    End If
                End If
            End If

        Catch ex As Exception
        End Try

        Return strNextLink

    End Function












    Private Function CreateAuthorisation(ByVal strUserName As String, ByVal strPassword As String)

        Dim strAuthorisation As String

        'if we have a user name and password, then create the basic authorisation header apply
        If (Not strUserName Is Nothing) AndAlso _
        (Not strPassword Is Nothing) AndAlso _
        (strUserName.Length > 0) AndAlso _
        (strPassword.Length > 0) Then

            'base64 encode the username and password and append
            strAuthorisation = "Basic " & Base64Encode(strUserName & ":" & strPassword)
        Else
            strAuthorisation = ""

        End If

        Return strAuthorisation

    End Function

    Private Function Base64Encode(ByVal strSource As String) As String

        Dim intData() As Byte
        Dim strResult As String

        intData = System.Text.Encoding.ASCII.GetBytes(strSource)
        strResult = System.Convert.ToBase64String(intData)

        Return strResult

    End Function
End Class
